#define MAX_ED 100000
#include <stdlib.h>
#include "GRAPH.h"

link NEW(unsigned int v, link next)
{
  link x = malloc(sizeof *x);
  x->v = v; x->next = next;
  return x;
}

Graph GRAPHinit(unsigned int V)
{
  unsigned int v;
  Graph G = malloc(sizeof *G);
  G->V = V; G->E = 0;
  G->adj = malloc(V*sizeof(link));
  for(v = 0; v < V; v++) G->adj[v] = NULL;
  return G;
}

void GRAPHinsertE(Graph G, Edge e)
{
  unsigned int v = e.v, w = e.w;
  link x;
  if(v != w)
    {
      for(x = G->adj[v]; x != NULL; x = x->next)
	if(x->v == w) return;
      G->adj[v] = NEW(w, G->adj[v]);
      G->adj[w] = NEW(v, G->adj[w]);
      G->E++;
    }
}

/*insere um arco (Directed Edge) no Grafo*/
void GRAPHinsertDE(Graph G, Edge e)
{
  unsigned int v = e.v, w = e.w;
  link x;
  for(x = G->adj[v]; x != NULL; x = x->next)
    if(x->v == w) return;
  G->adj[v] = NEW(w, G->adj[v]);
  G->E++;
}

void GRAPHremoveE(Graph G, Edge e)
{
  unsigned int v = e.v, w = e.w;
  link t, a = NULL;
  for(t = G->adj[v]; t != NULL; t = t->next)
    {
      if(t->v == w)
	{
	  G->E--;
	  if(a)
	    a->next = t->next;
	  else
	    G->adj[v] = t->next;
	}
      a = t;
    }
  a = NULL;
  for(t = G->adj[w]; t != NULL; t = t->next)
    {
      if(t->v == v)
	{
	  if(a)
	    a->next = t->next;
	  else
	    G->adj[w] = t->next;
	}
      a = t;
    }
}

unsigned int GRAPHedges(Edge a[], Graph G)
{
  unsigned int v, E = 0; link t;
  for(v = 0; v < G->V; v++)
    for(t = G->adj[v]; t != NULL; t = t->next)
      if(v < t->v) a[E++] = EDGE(v, t->v);
  return E;
      
}

Graph GRAPHcopy(Graph G)
{
  unsigned int i, n;
  Edge *a = malloc(MAX_ED*sizeof(Edge));
  Graph N = GRAPHinit(G->V);
  n = GRAPHedges(a, G);
  for(i = 0; i < n; i++)
    GRAPHinsertE(G,a[i]);
  free(a);
  return N;
}

void GRAPHdestroy(Graph G)
{
  unsigned int v; link atual, prox;
  for(v = 0; v < G->V; v++)
    {
      atual = G->adj[v];
      while(atual != NULL)
	{
	  prox = atual->next;
	  free(atual);
	  atual = prox;
	}
    }
  free(G->adj);
  free(G);
}

Graph GRAPHreverse(Graph G)
{
  int v; link t;
  Graph R = GRAPHinit(G->V);
  for (v = 0; v < G->V; v++)
    for(t = G->adj[v]; t != NULL; t = t->next)
      GRAPHinsertDE(R,EDGE(t->v, v));
  return R;
}

int decrescente(const void *a, const void *b)
{
  int *aa = (int *)a;
  int *bb = (int *)b;
  return *bb - *aa;
}
